/*
 * module to access the stack frame of all Python interpreter threads
 *
 * works on Solaris, portability to other OSes unknown
 *
 * Fazal Majid, 2002-10-11
 *
 * Copyright (c) 2002 Kefta Inc.
 * All rights reserved
 *
 */

#include "Python.h"
#include "compile.h"
#include "frameobject.h"
#include "patchlevel.h"

static PyObject *
threadframe_threadframe(PyObject *self, PyObject *args)
{
	PyInterpreterState *interp;
	PyThreadState *tstate;
	PyFrameObject *frame;
	PyListObject *frames;

	frames = (PyListObject*) PyList_New(0);
	if (! frames) return NULL;

	/* Walk down the interpreters and threads until we find the one
	   matching the supplied thread ID. */
	for (interp = PyInterpreterState_Head(); interp != NULL;
	     interp = interp->next) {
		for(tstate = interp->tstate_head; tstate != NULL;
		    tstate = tstate->next) {
			frame = tstate->frame;
			if (! frame) continue;
			Py_INCREF(frame);
			PyList_Append((PyObject*) frames, (PyObject*) frame);
		}
	}

	return (PyObject*) frames;
}

static char threadframe_doc[] =
"returns a list of frame objects for all threads, in no specific order.";

/* List of functions defined in the module */

static PyMethodDef threadframe_methods[] = {
	{"threadframe", threadframe_threadframe, METH_VARARGS,
	 threadframe_doc},
	{NULL,		NULL}		/* sentinel */
};


/* Initialization function for the module (*must* be called initthreadframe) */

static char module_doc[] =
"Debugging module to extract stack frames for all Python interpreter heads.\n"
"Useful in conjunction with traceback.print_stack().";

DL_EXPORT(void)
initthreadframe(void)
{
	PyObject *m;

	/* Create the module and add the functions */
	m = Py_InitModule3("threadframe", threadframe_methods, module_doc);
}
